;------------------------------------------------------------------------------
; COLOUR BITMAP COLOUR CYCLING TESTBED
; Code Copyright (C)2003 Andrew Davie -- atari2600@taswegian.com
; May be freely used for private, educational and research purposes.
; If you wish to profit from it, please ask me for permission first.
; Distributed to the [stella] list 30/March/2003


                processor 6502
                include "vcs.h"
                include "macro.h"

;ASYMMETRICAL           ; define for asymmetrical PF
DIRECTION = 0           ; 0 = inwards,  non-0 = outwards
;------------------------------------------------------------------------------

                SEG.U variables
                ORG $80

LineR0          ds 2                    ; 6 columns of RED sub-image
LineR1          ds 2
LineR2          ds 2
LineR3          ds 2
LineR4          ds 2
LineR5          ds 2

LineG0          ds 2                    ; 6 columns of GREEN sub-image
LineG1          ds 2
LineG2          ds 2
LineG3          ds 2
LineG4          ds 2
LineG5          ds 2

LineB0          ds 2                    ; 6 columns of BLUE sub-image
LineB1          ds 2
LineB2          ds 2
LineB3          ds 2
LineB4          ds 2
LineB5          ds 2

Phase           ds 1
Colour          ds 3

Count           ds 1


;------------------------------------------------------------------------------

            SEG code
            ORG $F000

    ; The image file is composed of strips (corresponding to PF0-PF2, repeated)
    ; which are decomposed by a utility from colour-separated RGB components of
    ; an original image. The displayed image is 40 pixels x 192 pixels.

    ; For access to the conversion/decomposition tool which generated these data
    ; please contact me privately.

        include "testblue.asm"
        include "testred.asm"
        include "testgreen.asm"

;------------------------------------------------------------------------------


Reset
            sei
            cld
               
		    ldx #$FF
		    txs 				; Clear the stack

            lda #RED
            sta Colour
            lda #GREEN
            sta Colour+1
            lda #BLUE
            sta Colour+2


    ; Start of vertical blank processing


NewFrame    lda #2
            sta WSYNC

            sta VBLANK          

            sta VSYNC           
            sta WSYNC           
            sta WSYNC
            sta WSYNC

            lda #0
            sta VSYNC           

            lda #43             
            sta TIM64T

            lda #0
            sta COLUBK

VblankLoop  lda INTIM
            bne VblankLoop      
            
            sta WSYNC           
            sta VBLANK          

    ;------------------------------------------------------------------------------
    ; START OF DISPLAY


            lda #238                ; NTSC
            sta TIM64T

    #ifconst ASYMMETRICAL
            lda #%00010100      ; reflect, priority, 2 pixel wide ball
    #else
            lda #%00010101      ; NO reflect, priority, 2 pixel wide ball
    #endif
            sta CTRLPF
          
            lda #0
            sta PF0
            sta PF1
            sta PF2

    ;--------------------------------------------------------------------------
    ; START OF COLOUR BITMAP DISPLAY KERNEL


RED         = $D0 ;%00110100              ; muck around with these and see what happens
GREEN       = $D6
BLUE        = $D2 ;%01110000


            lda #<testred_STRIP_0
            sta LineR0
            lda #>testred_STRIP_0
            sta LineR0+1

            lda #<testred_STRIP_1
            sta LineR1
            lda #>testred_STRIP_1
            sta LineR1+1

            lda #<testred_STRIP_2
            sta LineR2
            lda #>testred_STRIP_2
            sta LineR2+1

    #ifconst ASYMMETRICAL

            lda #<testred_STRIP_3
            sta LineR3
            lda #>testred_STRIP_3
            sta LineR3+1

            lda #<testred_STRIP_4
            sta LineR4
            lda #>testred_STRIP_4
            sta LineR4+1

            lda #<testred_STRIP_5
            sta LineR5
            lda #>testred_STRIP_5
            sta LineR5+1
    #endif


            lda #<testgreen_STRIP_0
            sta LineG0
            lda #>testgreen_STRIP_0
            sta LineG0+1

            lda #<testgreen_STRIP_1
            sta LineG1
            lda #>testgreen_STRIP_1
            sta LineG1+1

            lda #<testgreen_STRIP_2
            sta LineG2
            lda #>testgreen_STRIP_2
            sta LineG2+1

    #ifconst ASYMMETRICAL

            lda #<testgreen_STRIP_3
            sta LineG3
            lda #>testgreen_STRIP_3
            sta LineG3+1

            lda #<testgreen_STRIP_4
            sta LineG4
            lda #>testgreen_STRIP_4
            sta LineG4+1

            lda #<testgreen_STRIP_5
            sta LineG5
            lda #>testgreen_STRIP_5
            sta LineG5+1
    #endif


            lda #<testblue_STRIP_0
            sta LineB0
            lda #>testblue_STRIP_0
            sta LineB0+1

            lda #<testblue_STRIP_1
            sta LineB1
            lda #>testblue_STRIP_1
            sta LineB1+1

            lda #<testblue_STRIP_2
            sta LineB2
            lda #>testblue_STRIP_2
            sta LineB2+1

    #ifconst ASYMMETRICAL

            lda #<testblue_STRIP_3
            sta LineB3
            lda #>testblue_STRIP_3
            sta LineB3+1

            lda #<testblue_STRIP_4
            sta LineB4
            lda #>testblue_STRIP_4
            sta LineB4+1

            lda #<testblue_STRIP_5
            sta LineB5
            lda #>testblue_STRIP_5
            sta LineB5+1

    #endif


            clc
            lda Phase
            adc #1
            cmp #3
            bcc PhaseOK
            lda #0
PhaseOK     sta Phase


    ; Now cycle the colours

            inc Count
            lda Count
            cmp #10                 ; this controls cycling speed - change as you wish
            bcc Nzap

    #if DIRECTION = 0   ;inwards

            ldx Colour
            lda Colour+2
            sta Colour
            lda Colour+1
            sta Colour+2
            stx Colour+1

    #else
            ldx Colour
            lda Colour+1
            sta Colour
            lda Colour+2
            sta Colour+1
            stx Colour+2

    #endif


            lda #0
            sta Count

Nzap


            
            ldy #191                ; total number of scanlines to display


    ; Vector to the correct start - a red, blue or green line

            ldx Phase
            beq ScanRed
            dex
            beq ScanGreen
            bne ScanBlue

ScanRed     sta WSYNC

            lda Colour
            sta COLUPF

            lda (LineR0),y
            sta PF0
            lda (LineR1),y
            sta PF1
            lda (LineR2),y
            sta PF2

    #ifconst ASYMMETRICAL

            lda (LineR3),y
            sta PF0
            lda (LineR4),y
            sta PF1
            lda (LineR5),y
            sta PF2
    #endif


            dey
            beq ScanEnd

          
ScanGreen   STA WSYNC

            lda Colour+1
            sta COLUPF

            lda (LineG0),y
            sta PF0
            lda (LineG1),y
            sta PF1
            lda (LineG2),y
            sta PF2

    #ifconst ASYMMETRICAL
            lda (LineG3),y
            sta PF0
            lda (LineG4),y
            sta PF1
            lda (LineG5),y
            sta PF2
    #endif

            dey
            beq ScanEnd


ScanBlue    sta WSYNC
                
            lda Colour+2
            sta COLUPF

            lda (LineB0),y
            sta PF0
            lda (LineB1),y
            sta PF1
            lda (LineB2),y
            sta PF2

    #ifconst ASYMMETRICAL

            lda (LineB3),y
            sta PF0
            lda (LineB4),y
            sta PF1
            lda (LineB5),y
            sta PF2
    #endif

            dey
            bne ScanRed

ScanEnd

    ; Screen has been drawn - now just tidy up

            sty WSYNC
            sty PF0
            sty PF1
            sty PF2

PadScreen2  lda INTIM
            bne PadScreen2
  
            sta WSYNC

            lda #%01000010
            sta VBLANK                      ; end of screen - enter blanking

            lda #25
            sta TIM64T

Overscan    lda INTIM
            bne Overscan

            sta WSYNC           
            sta WSYNC           

            jmp NewFrame

    ;--------------------------------------------------------------------------

    ORG $fffa 

            .word Reset           ; NMI
            .word Reset           ; RESET
            .word Reset           ; IRQ


